package cn.jlsxwkj.common.filter;

import cn.jlsxwkj.common.config.UserConfig;
import cn.jlsxwkj.common.utils.Log;
import jakarta.annotation.Resource;
import jakarta.servlet.*;
import jakarta.servlet.http.HttpServletRequest;
import org.springframework.stereotype.Component;

import java.io.IOException;

/**
 * @author zh
 */
@Component
public class ExecTimeFilter implements Filter {

    /**
     * 用户配置.
     */
    @Resource
    private UserConfig userConfig;

    /**
     * @param servletRequest  ServletRequest
     * @param servletResponse ServletResponse
     * @param filterChain     FilterChain
     * @throws IOException      IOException
     * @throws ServletException ServletException
     */
    @Override
    public void doFilter(
            final ServletRequest servletRequest,
            final ServletResponse servletResponse,
            final FilterChain filterChain)
            throws IOException, ServletException {
        long startTime = System.currentTimeMillis();
        String url = ((HttpServletRequest) servletRequest)
                .getRequestURL().toString();
        filterChain.doFilter(servletRequest, servletResponse);
        long execTime = System.currentTimeMillis() - startTime;
        if (execTime > userConfig.getExecTime()) {
            Log.warn(this.getClass(),
                    "api: {}, exec_time ====> {}ms", url, execTime);
        }
    }

}
